function [f,fd1,fd2] = evalbase(x,n,a,b,c,base);

% EVALBASE: [f,fd1,fd2] = evalbase(x,n,a,b,c,base);
%
% Evaluate linear combinations of multivariate basis functions
%
% Given m by d matrix x containing m vectors in R^d, returns the
% m by 1 vector of function values f, the m by d vector of first
% partial derivatives fd1, and the m by d vector of second own
% own partial derivatives fd2 for a real-valued function on R^d
% evaluated at the m points, where the function is represented
% as a linear combination of either Chebychev (base=basecheb),
% cubic spline (base=basecspl), or linear spline (base=baselspl)
% multivariate basis functions. The orders and endpoints of the
% basis functions are given by the d by 1 vectors n, a, and b,
% and the basis function coefficients are given by the prod(n) by
% 1 vector c.

[m,d] = size(x);

if d==1
   switch nargout
      case 1, phi = feval(base,x,n,a,b);
      case 2, [phi,phider1] = feval(base,x,n,a,b);
      case 3, [phi,phider1,phider2] = feval(base,x,n,a,b);
   end
   f = phi*c;
   if nargout>1, fd1 = phider1*c; end
   if nargout>2, fd2 = phider2*c; end
   return
end

switch nargout
   case 1
      for k=1:d
         phi{k} = feval(base,x(:,k),n(k),a(k),b(k));
      end
   case 2
      for k=1:d
         [phi{k},phider1{k}] = feval(base,x(:,k),n(k),a(k),b(k));
      end
   case 3
      for k=1:d
         [phi{k},phider1{k},phider2{k}] = feval(base,x(:,k),n(k),a(k),b(k));
      end
end

f = kronrow(phi,c);

if nargout>1
   fd1 = zeros(m,d);
   for id=1:d
      phitmp = phi{id};
      phi{id} = phider1{id};
      fd1(:,id) = kronrow(phi,c);
      phi{id} = phitmp;
   end
end

if nargout>2
   fd2 = zeros(m,d);
   for id=1:d
      phitmp = phi{id};
      phi{id} = phider2{id};
      fd2(:,id) = kronrow(phi,c);
      phi{id} = phitmp;
   end
end
